package com.huojutech.controller.weixin;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.huojutech.constant.SessionConstants;
import com.huojutech.dto.CountDTO;
import com.huojutech.dto.SeerListDTO;
import com.huojutech.dto.UserInfoDTO;
import com.huojutech.entity.User;
import com.huojutech.entity.UserConcern;
import com.huojutech.entity.UserInvite;
import com.huojutech.service.ForecastService;
import com.huojutech.service.StockOptionalService;
import com.huojutech.service.UserConcernService;
import com.huojutech.service.UserInviteService;
import com.huojutech.service.UserService;
import com.huojutech.utils.AesCbcUtil;
import com.huojutech.utils.ResultEnum;
import com.huojutech.vo.PageResultVO;
import com.huojutech.vo.ResultVO;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * Created by Wangzw on 2018\4\16 0016.
 */
@RestController
@Api(description = "用户中心接口")
@RequestMapping("/wx/userCenter")
public class UserCenterController {

    private static Logger logger = LoggerFactory.getLogger(UserCenterController.class);

    @Autowired
    private UserConcernService userConcernService;

    @Autowired
    private StockOptionalService stockOptionalService;

    @Autowired
    private UserService userService;

    @Autowired
    private ForecastService forecastService;

    @Autowired
    private UserInviteService userInviteService;

    @Autowired
    private RedisTemplate redisTemplate;

    @RequestMapping(value = "/info", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
    @ApiOperation(value = "查询用户中心所有立即展示的数据", notes = "通过用户id查询", response = CountDTO.class)
    public ResultVO<CountDTO> myUserCenterNum() throws Exception {
        Integer uid = SessionConstants.getSession();
        Integer concernCounts =  userConcernService.selectConcernCounts(uid);//用户关注数量
        Integer fansCounts = userConcernService.selectFansCounts(uid);//用户粉丝数量
        Integer optionalCounts = stockOptionalService.selectOptionalCounts(uid);//自选股数量查询
        User user = userService.selectUserById(uid);
        logger.info("用户id是："+user.getId());
        Double forecastRateSuccess = user.getForecastRateSuccess();//预测成功率
        Double voteRateSuccess = user.getVoteRateSuccess();//投票成功率
        String nickname = user.getNickname();
        String avatar = user.getAvatar();
        Integer forecastCounts = user.getFnum();//预测数量查询
        Integer voteCounts = user.getVoteNum();//投票数量查询

        //小数转百分数
        DecimalFormat df = new DecimalFormat("0.00%");
        String forecastSuc = df.format(forecastRateSuccess);
        String voteRateSuc = df.format(voteRateSuccess);

        BigDecimal amount = user.getAmount();//拥有火钻数量
        String stringAmount = amount.toString();


        CountDTO countDTO = new CountDTO();
        countDTO.setUserConcernNum(concernCounts);
        countDTO.setFansNum(fansCounts);
        countDTO.setForecastNum(forecastCounts);
        countDTO.setVoteNum(voteCounts);
        countDTO.setOptionalNum(optionalCounts);
        countDTO.setDiamondNum(stringAmount);
        countDTO.setForecastSuc(forecastSuc);
        countDTO.setVoteSuc(voteRateSuc);
        countDTO.setAvatar(avatar);
        countDTO.setNickName(nickname);

        ResultVO<CountDTO> vo = new ResultVO<>();
        vo.setData(countDTO);
        vo.setCode(ResultEnum.SUCCESS.getCode());
        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        return vo;

    }

    @RequestMapping(value = "/fans/List", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
    @ApiOperation(value = "查询本用户的粉丝列表", notes = "通过用户id查询", response = CountDTO.class)
    public PageResultVO<List<SeerListDTO>> fans(
            @ApiParam(value="分页大小,默认10",required=false) @RequestParam(required=false,defaultValue="10") Integer size,
            @ApiParam(value="页码 ，第一页为1") @RequestParam(required=false,defaultValue="1") Integer page) throws Exception {
        Integer uid = SessionConstants.getSession();
        List<SeerListDTO> seerListDTOS = new ArrayList<>();
        PageHelper.startPage(page,size);
        List<UserConcern> fansList = userConcernService.selectFansList(uid);
        getPerson(seerListDTOS,fansList);//获取粉丝抽取
        PageResultVO<List<SeerListDTO>> vo = new PageResultVO<>();
        vo.setData(seerListDTOS);

        vo.setCode(ResultEnum.SUCCESS.getCode());
        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        return vo;
    }

    @RequestMapping(value = "/concerns/List", produces = "application/json;charset=utf-8", method = RequestMethod.GET)
    @ApiOperation(value = "查询本用户的粉丝列表", notes = "通过用户id查询", response = CountDTO.class)
    public PageResultVO<List<SeerListDTO>> concern(
            @ApiParam(value="分页大小,默认10",required=false) @RequestParam(required=false,defaultValue="10") Integer size,
            @ApiParam(value="页码 ，第一页为1") @RequestParam(required=false,defaultValue="1") Integer page) throws Exception {
        Integer uid = SessionConstants.getSession();
        List<SeerListDTO> seerListDTOS = new ArrayList<>();
        PageHelper.startPage(page,size);
        List<UserConcern> concernList = userConcernService.selectConcernList(uid);
        getPerson(seerListDTOS, concernList);//获取关注抽取
        PageResultVO<List<SeerListDTO>> vo = new PageResultVO<>();
        vo.setData(seerListDTOS);

        vo.setCode(ResultEnum.SUCCESS.getCode());
        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        return vo;
    }

    /**
     * 获取用户粉丝和关注抽取方法
     * @param seerListDTOS
     * @param concernList
     */
    private void getPerson(List<SeerListDTO> seerListDTOS, List<UserConcern> concernList) {
        for (UserConcern list:concernList) {
            SeerListDTO seerListDTO = new SeerListDTO();
            Integer uid1 = list.getUid();
            User user = userService.selectUserById(uid1);
            String nickname = user.getNickname();//昵称
            String avatar = user.getAvatar();//头像
            Double forecastRateSuccess = user.getForecastRateSuccess();

            DecimalFormat df = new DecimalFormat("0.00%");
            String forecastSuc = df.format(forecastRateSuccess);//成功率

            Integer forecastNum = forecastService.findForecastNumByUid(uid1);//预测数量
            Integer concernCounts = userConcernService.selectFansCounts(uid1);//别人关注他数量

            seerListDTO.setId(uid1);
            seerListDTO.setAvatar(avatar);
            seerListDTO.setNickname(nickname);
            seerListDTO.setForecastSuc(forecastSuc);
            seerListDTO.setConcernNum(concernCounts);
            seerListDTO.setForecastCounts(forecastNum);

            seerListDTOS.add(seerListDTO);
        }
    }

    @SuppressWarnings("rawtypes")
    @RequestMapping(value = "/sign/info", produces = "application/json;charset=utf-8", method = RequestMethod.POST)
    @ApiOperation(value = "用户授权时获取用户信息", notes = "用户授权时获取用户信息")
    public ResultVO signInfo(@RequestBody UserInfoDTO userInfoDTO) {
        Integer sessionId = SessionConstants.getSession();
        ResultVO vo = new ResultVO();
        if (userInfoDTO != null) {
            String nickname = userInfoDTO.getNickname();
            String avatar = userInfoDTO.getAvatar();
            User user = new User();
            user.setId(sessionId);
            user.setNickname(nickname);
            user.setAvatar(avatar);

            ValueOperations valueOperations = redisTemplate.opsForValue();
            String sessionKey = (String) valueOperations.get("sessionKey");
            String result = null;
            try {
                result = AesCbcUtil.decrypt(userInfoDTO.getEncryptedData(),sessionKey, userInfoDTO.getIv(), "UTF-8");
            } catch (Exception e) {
                logger.info(e.getMessage(), e);
            }
            if (null != result && result.length() > 0) {
                JSONObject userInfoJSON = JSON.parseObject(result);
                String unionId = (String) userInfoJSON.get("unionId");
                user.setWxUnionid(unionId);
            }
            userService.updateUser(user);
        }

        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        vo.setCode(ResultEnum.SUCCESS.getCode());
        return vo;
    }
    @SuppressWarnings("rawtypes")
    @RequestMapping(value = "/invite/{id}", produces = "application/json;charset=utf-8", method = RequestMethod.POST)
    @ApiOperation(value = "邀请用户", notes = "邀请用户")
    public ResultVO invite(@ApiParam(value="邀请者的id") @PathVariable(value="id") Integer id) {
        Integer sessionId = SessionConstants.getSession();
        ResultVO vo = new ResultVO();
        if (id!=null){
            //如果用户不存在，是盗刷连接
            User user = userService.selectUserById(id);
            if (user==null){
                vo.setMsg(ResultEnum.FAILED.getMessage());
                vo.setCode(ResultEnum.FAILED.getCode());
                return vo;
            }
            //如果id是自己，自己不能邀请自己
            if (sessionId.equals(id)){
                vo.setMsg(ResultEnum.FAILED.getMessage());
                vo.setCode(ResultEnum.FAILED.getCode());
                return vo;
            }
            //如果自己已经被邀请过了，此邀请不能不算数
            List<UserInvite> inviteUser = userInviteService.findInfoByInviteId(sessionId);
            if (inviteUser.size()>0){
                vo.setMsg(ResultEnum.REPEAT_INVITE.getMessage());
                vo.setCode(ResultEnum.REPEAT_INVITE.getCode());
                return vo;
            }
            UserInvite userInvite = new UserInvite();
            userInvite.setUid(id);
            userInvite.setInviteId(sessionId);
            userInviteService.insertUser(userInvite);
        }
        vo.setMsg(ResultEnum.SUCCESS.getMessage());
        vo.setCode(ResultEnum.SUCCESS.getCode());
        return vo;
    }
}
